@@ -17,13 +17,12 @@ LoadModule <- function(module){
   
   # module must be a string 
   
+  # URL to module in user's favourite repo & branch
+  zoonURL <- sprintf('%s/%s/R/%s.R',
+                     options('zoonRepo'),
+                     options('zoonRepoBranch'),
+                     module)
   
-  # Create url that matches zoon repo
-  # .sha will vary based on whether this is pegged to a specific version of
-  # modules
-  zoonURL <- 
-    sprintf('https://raw.githubusercontent.com/zoonproject/modules/%s/R/%s.R',
-            .sha, module)
   # If module is a path, load module
   if (file.exists(module)){
     txt <- parse(text = paste(readLines(module), collapse="\n"))
@@ -70,10 +69,13 @@ LoadModule <- function(module){
 #@name GetModule
 
 GetModule <- function(module, forceReproducible){
-  zoonURL <- 
-    paste0('https://raw.githubusercontent.com/zoonproject/modules/master/R/',
-           module, '.R')
   
+  # URL to module in user's favourite repo & branch
+  zoonURL <- sprintf('%s/%s/R/%s.R',
+                     options('zoonRepo'),
+                     options('zoonRepoBranch'),
+                     module)
+
   # If the module is in global namespace, use that function
   #   unless forceReproduce is TRUE, in which case we want to get from repo.
   #   
@@ -156,8 +158,18 @@ RunModels <- function(df, modelFunction, paras, workEnv){
   k <- length(unique(df$fold)[unique(df$fold) != 0])
   
   # Init. output dataframe with predictions column
-  dfOut <- cbind(df[, 1:5], predictions = NA, df[,6:NCOL(df)])
-  names(dfOut)[7:ncol(dfOut)] <- names(df)[6:ncol(df)]
+  # Old versions of modules dont use this attribute 
+  ## REMOVE ONCE MODULES UPDATED ##
+  if('covCols' %in% names(attributes(df))){
+    dfOut <- cbind(df[!colnames(df) %in% attr(df, 'covCols')],
+                   predictions = NA,
+                   df[colnames(df) %in% attr(df, 'covCols')])
+  } else {
+    
+    dfOut <- cbind(df[, 1:5], predictions = NA, df[,6:NCOL(df)])
+    names(dfOut)[7:ncol(dfOut)] <- names(df)[6:ncol(df)]
+    
+  }
   
   # We don't know that they want cross validation.
   # If they do, k>1, then run model k times and predict out of bag
@@ -168,8 +180,17 @@ RunModels <- function(df, modelFunction, paras, workEnv){
                                             paras),
                            envir = workEnv)
       
-      pred <- ZoonPredict(modelFold,
-                          newdata = df[df$fold == i, 6:NCOL(df), drop = FALSE])
+      # Old versions of modules dont use this attribute 
+      ## REMOVE ONCE MODULES UPDATED ##
+      if('covCols' %in% names(attributes(df))){
+        pred <- ZoonPredict(modelFold,
+                            newdata = df[df$fold == i,
+                                         attr(df, 'covCols'),
+                                         drop = FALSE])
+      } else {
+        pred <- ZoonPredict(modelFold,
+                            newdata = df[df$fold == i, 6:NCOL(df), drop = FALSE])
+      }
       
       dfOut$predictions[df$fold == i] <- pred 
       
@@ -183,8 +204,17 @@ RunModels <- function(df, modelFunction, paras, workEnv){
   # If external validation dataset exists, predict that;.
   if(0 %in% df$fold){
     
-    pred <- ZoonPredict(m,
-                        newdata = df[df$fold == 0, 6:NCOL(df), drop = FALSE])
+    # Old versions of modules dont use this attribute 
+    ## REMOVE ONCE MODULES UPDATED ##
+    if('covCols' %in% names(attributes(df))){
+      pred <- ZoonPredict(m,
+                          newdata = df[df$fold == 0,
+                                       attr(df, 'covCols'),
+                                       drop = FALSE])
+    } else {
+      pred <- ZoonPredict(m,
+                          newdata = df[df$fold == 0, 6:NCOL(df), drop = FALSE])
+    }
     
     dfOut$predictions[df$fold == 0] <- pred 
     
@@ -218,7 +248,7 @@ CheckModList <- function(x){
   # Should accept occurrence = 'module1', but NOT 
   #   occurrence = 'module1(k=2)', or occurrence = 'list(mod1, mod1)'
   if (inherits(x, 'character')){
-    if (grepl('[!#$%&*+,-/:;<>?@[\ ]^_`| ]', x)){
+    if (grepl("[^\' | ^\"]", x) & grepl("[( | )]", x)){
       stop(paste('If specifying module arguments please use the form',
                  'Module(para = 2), without quotes. No special characters should exist',
                  'in module names.'))
@@ -244,6 +274,13 @@ CheckModList <- function(x){
     ModuleList <- lapply(listCall, FormatModuleList) 
     attr(ModuleList, 'chain') <- TRUE
     
+    # If unquoted module w/ paras given: occurrence = Module1(k=2)
+  } else if (x[[1]] == 'Replicate'){
+    
+    listCall <- eval(x)
+    
+    ModuleList <- lapply(listCall, FormatModuleList) 
+
     # If unquoted module w/ paras given: occurrence = Module1(k=2)
   } else if (identical(class(x[[1]]), 'name')){
     # Parameters
@@ -324,6 +361,16 @@ FormatModuleList <- function(x){
 
 ExtractAndCombData <- function(occurrence, ras){
   
+  # Check for rows in the occurrence data that have missing data
+  NArows <- apply(occurrence[,c('longitude', 'latitude')],
+                  1,
+                  function(x) any(is.na(x)))
+  
+  if(any(NArows)){
+    warning(sum(NArows), ' row(s) of occurrence data have NA values for latitude/longitude and will be removed')
+    occurrence <- occurrence[!NArows, ]
+  }
+  
   # Check that all points are within the raster
   bad.coords <- is.na(cellFromXY(ras,
                                  occurrence[,c('longitude', 'latitude')]))
@@ -334,11 +381,16 @@ ExtractAndCombData <- function(occurrence, ras){
   
   # extract covariates from lat long values in df.
   occurrenceCovariates <- as.matrix(raster::extract(ras, occurrence[, c('longitude', 'latitude')]))
-  names(occurrenceCovariates) <- names(ras)  
+  colnames(occurrenceCovariates) <- names(ras)  
   
   # combine with the occurrence data
   df <- cbind(occurrence, occurrenceCovariates)
   
+  # assign call_path attribute to this new object
+  attr(df, 'call_path') <- attr(occurrence, 'call_path')
+  
+  # record the covariate column names
+  attr(df, 'covCols') <- names(ras)
   
   # Return as list of df and ras as required by process modules
   return(list(df=df, ras=ras))
@@ -615,4 +667,4 @@ GetModuleVersion <- function(rawText){
   Start <- TagPosition[[1]] + attr(TagPosition[[1]], 'match.length')
   substr(VersionLine, Start, nchar(VersionLine))    
 
-}
\ No newline at end of file
+}
